#!/usr/bin/python
# -*- coding: utf-8 -*-

# BotMUUpdater.py
#
# This file is part of BotMU
#
# Author: Raphaël Perraudeau <r.perraudeau@gmail.com>
# Copyright: 2011 Raphaël Perraudeau <r.perraudeau@gmail.com>
# Licence: GNU/GPL
#
# Home page: <http://botmu.sourceforge.net/>

"""python BotMUUpdater.py --link"""


# Inclusions des modules générique
# Native
import sys
import platform
import time
import os
import subprocess


# Pyqt4
from PyQt4.QtCore import *
from PyQt4.QtGui import *

from httpReq import *

#############################################################################
class Abort(Exception):
	"""classe d'exception crée pour l'arrêt du téléchargement avant la fin"""
	pass


#############################################################################
class MainWindow(QWidget):

	#========================================================================
	def __init__(self, app):
		QMainWindow.__init__(self)

		self.resize(350, 100)
		self.setWindowTitle('Mise à jour de BotMU')
		self.setWindowIcon(QIcon("BotMU16.ico"))

		#Variables
		self.verbose = False
		self.proxy = ''
		self.telech = None
		self.execTime = 0.0
		self.downloadedOctetsInThreeSecond = 0
		self.lasttime = 0
		self.downloadSpeed = 0
		self.timeLeft = "?"
		self.barreNull = 1

		# Linux:
		if platform.system() == 'Linux':
			self.configDir			= "./"

		# Windows: Fonctionne avec XP, à tester avec Vista/Seven
		if platform.system() == 'Windows':
			self.configDir			= os.environ['APPDATA']

		# FIXME Mac
		else:
			self.configDir			= "./"
		
		
		#Stop
		self.stop = QPushButton(QIcon('icons/stop.png'), u"  Stop", self)
		
		#Label d'information
		self.labelInfo = QLabel(u"", self)
		self.labelInfo.setText("Contact du site...")
		self.labelInfo.setAlignment(Qt.AlignCenter)
		
		#Barre de progression
		self.barre = QProgressBar(self)
		self.barre.setRange(0, 0)
		self.barre.setValue(0)
		
		#Positionnement
		posit = QGridLayout()
		posit.addWidget(self.labelInfo, 0, 0, 1, 2)
		posit.addWidget(self.barre, 1, 0, 1, 2)
		posit.addWidget(self.stop, 2, 1)
		self.setLayout(posit)
		
		
		if(len(sys.argv)>1):
			self.telech = Telecharger(sys.argv[1], parent=self)
		
		#Connexions
		self.stop.clicked.connect(self.close)
		if(len(sys.argv)>1):
			self.connect(self.telech, SIGNAL("infotelech(PyQt_PyObject)"), self.infotelech)
			self.connect(self.telech, SIGNAL("fintelech(PyQt_PyObject)"), self.fintelech)
			self.telech.start()
		else:
			sys.exit()

	# Event de fermeture
	#========================================================================
	def closeEvent(self, event):
		"""lancé à la fermeture de la fenêtre quelqu'en soit la méthode"""
		
		reply = QMessageBox.warning(self, "Quitter ?", u"Annuler la mise à jour de BotMU ?", QMessageBox.Yes|QMessageBox.No)
		if(reply==QMessageBox.Yes):
			event.accept()
			sys.exit()
		else:
			event.ignore()

	# lecture du fichier de configuration
	#========================================================================
	def readOptions(self):
		setupFile = os.path.join(self.configDir, "setup.ini")
		paramfile = open(setupFile)
		param = (paramfile.read()).split("\n")
		for item in param:
			if item.split("=")[0] == 'proxy':
				self.proxy = item.split("=")[1]
			if item.split("=")[0] == 'verbose':
				if item.split("=")[1] == 'True':
					self.verbose = True
				else:
					self.verbose = False
					
	# Récéption et affichage d'information du téléchargement
	#========================================================================
	def infotelech(self, msg):
		"""lancé à chaque réception d'info sur la progression du téléchargement"""
		telechbloc, taillebloc, totalblocs, msgInf = msg
		if totalblocs > 0:
			#calcul et affichage de la progression
			p = int(float(telechbloc)*float(taillebloc)/float(totalblocs)*100)
			if self.barreNull == 1:
				self.barre.setRange(0, 100)
				self.barreNull = 0
			self.barre.setValue(p)
			QCoreApplication.processEvents() # force le rafraichissement
			
		else:
			p = 0
			timeLeft = 0
			# taille maxi inconnue: la barre sera une chenille sans progression
			if self.barre.maximum > 0:
				self.barre.reset()
				self.barre.setRange(0, 0)
				self.barreNull = 1
		textToShow = u"%s" % (msgInf[0])
		self.labelInfo.setText(textToShow)

					
	#========================================================================
 	def fintelech(self, msg):
		"""Lancé quand le thread se termine (normalement ou pas)"""
		self.barre.setValue(100)
		if(msg!="error" and msg!="cancel"):
			if(platform.system()=="Windows"):
				#os.system(msg[0])
				subprocess.Popen(msg[0])
				sys.exit()
			else:
				QMessageBox.information(self, u"Mise à jour automatique", u"La mise à jour automatique à échouée\n\nVous pouvez exécuter cette mise à jour manuellement,\nl'archive d'installation se trouve dans \"/tmp\"")
				sys.exit()


#############################################################################
class Telecharger(QThread):
	"""Thread de téléchargement"""

	#========================================================================
	def __init__(self, link, parent=None):
		super(Telecharger,self).__init__(parent)
		self.parent = parent
		self.link = link
		self.stop = False
	
	#========================================================================
	def run(self):
		httpreq = httpReq()
		httpreq.defineUseragent(platform.system())
		httpreq.defineProxy(self.parent.proxy)
		httpreq.defineVerbose(int(self.parent.verbose))
		filename = unicode((self.link.split("/")[-2:-1])[0])
		if("%2F" in filename):
			filename = filename.split("%2F")[-1]
		if(platform.system()=="Windows"):
			dest = "C:\WINDOWS\Temp\%s" % filename
		else:
			dest = "/tmp/%s" % filename
		try:
			self.msgInf = [u"Téléchargement de %s en cours" % (filename), 2]
			page = httpreq.getPage(self.link, dest=dest, report_hook=self.infotelech)
			messagefin = dest
		except Abort:
			messagefin = u"cancel"
		except:
			messagefin = u"error"
		# fin du téléchargement: émission du message de fin
		self.emit(SIGNAL("fintelech(PyQt_PyObject)"), [messagefin])

	#========================================================================
	def infotelech(self, telechbloc, taillebloc, totalblocs):
		"""reçoit les infos de progression du téléchargement"""
		# nécessaire pour stopper le téléchargement avant la fin
		if self.stop:
			raise Abort
		# envoie les infos de progression à la fenêtre graphique
		self.emit(SIGNAL("infotelech(PyQt_PyObject)"), 
										  [telechbloc, taillebloc, totalblocs, self.msgInf])
 
	#========================================================================
	def stoptelech(self):
		# permet l'arrêt du téléchargement avant la fin
		self.stop = True



#############################################################################
if __name__ == "__main__":
	app = QApplication(sys.argv)
	fen = MainWindow(app)
	fen.setWindowTitle("Bot MegaUpload Updater")
	fen.show()
	sys.exit(app.exec_())
